from ..schema import Relation, ReadOnlyNode
from colander import SchemaNode, String, Boolean, Integer
from deform.widget import TextAreaWidget, SelectWidget
from gocept.directory2.db_domainshuttle import IDatabase
from gocept.directory2.domain import User
import colander
import pkg_resources
import sqlalchemy
import zope.component


class ReadOnlyUserSelectNode(colander.SchemaNode):
    "Schema node for a read only field selecting user of the current domain."

    def __init__(self, *args, **kw):

        @colander.deferred
        def user_widget(node, kw):
            # unfortunately deform does not provide a proper readonly template
            # for select widgets.
            template = pkg_resources.resource_filename(
                'gocept.directory2.admin.domain', 'templates/readonly_select')
            return SelectWidget(
                template=template,
                values=[(x.id, x.name) for x in User.query()])

        @colander.deferred
        def domain_user(node, kw):
            context = kw['request'].context
            if isinstance(context, User):
                return context
            return context.user

        return super(ReadOnlyUserSelectNode, self).__init__(
            Relation(User), title='User', missing=domain_user,
            default=domain_user, widget=user_widget)


class AddDomainSchema(colander.Schema):
    """Schema to add a domain."""

    name = SchemaNode(String(), title='Domainname')
    user = SchemaNode(Relation(User))
    idn_name = ReadOnlyNode(String(), title='IDN', missing='')
    httpd_conf = SchemaNode(Boolean(), title='Apache config',
                            default=True, missing=True)
    sendmail_cw = SchemaNode(Boolean(), title='Mail config',
                             default=True, missing=True)
    named_conf = SchemaNode(Boolean(), title='DNS config',
                            default=True, missing=True)
    rel_path = SchemaNode(String(), title='Relative path', missing='')
    cname = SchemaNode(String(), title='CNAME', missing='')
    ip = SchemaNode(String(), title='IP', missing='')
    primary_dns = SchemaNode(String(), title='Primary DNS',
                             default='ns1.gocept.com')
    second_dns = SchemaNode(String(), title='Secondary DNS',
                            default='ns2.gocept.com')
    freeRecords = SchemaNode(String(), title='Additional DNS Record',
                             missing='', widget=TextAreaWidget())


class AddDomainSchemaWithPreselectedUser(AddDomainSchema):

    user = ReadOnlyUserSelectNode()


class AddSubDomainSchema(AddDomainSchema):

    user = ReadOnlyUserSelectNode()


class EditDomainSchema(AddDomainSchema):

    name = ReadOnlyNode(String(), title='Domainname')
    user = ReadOnlyUserSelectNode()


class AddUserSchema(colander.Schema):
    """Schema to add a user."""

    @colander.deferred
    def suggested_uid(node, kw):
        session = zope.component.getUtility(IDatabase).session
        max_uid = session.query(sqlalchemy.func.max(User.id)).first()[0]
        return max_uid + 1

    id = SchemaNode(Integer(), title='UID', default=suggested_uid)
    name = SchemaNode(String(), title='Name')
    customer_id = SchemaNode(Integer(), title='Customer ID', missing=0)
    realname = SchemaNode(String(), title='Real name', missing='')


class EditUserSchema(AddUserSchema):
    """Schema to edit a user."""

    id = ReadOnlyNode(Integer(), title='UID')
