from ..network import Network
from .view import View, Action
from colander import SchemaNode, String, Boolean
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form


@view_config(route_name='vlan_view',
             renderer='templates/vlan_view.pt')
class ViewVLAN(View):

    @property
    def title(self):
        return 'VLAN %s' % self.context.title

    @property
    def actions(self):
        yield Action('New network', 'icon-plus', 'network_add',
                     id=self.context.id)


@view_config(route_name='network_view',
             renderer='templates/network_view.pt')
class ViewNetwork(View):

    @property
    def title(self):
        return self.context.network

    @property
    def actions(self):
        yield Action('Edit', 'icon-edit', 'network_edit',
                     id=self.context.id)


class Schema(colander.Schema):

    network = SchemaNode(
        String(), title='Network',
        description='Please use a CIDR representation of the network.')
    dhcp_dynamic_range = SchemaNode(
        Boolean(), title='DHCP dynamic range',
        description='Is this network used for dynamic allocation via DHCP?')


@view_config(route_name='network_add')
class Add(gocept.directory2.admin.form.Add):

    title = 'Add Network'
    schema = Schema()
    factory = Network

    next_route = 'vlan_view'

    @property
    def next_route_kw(self):
        return {'id': self.context.id}

    def after_add(self, obj):
        try:
            self.context.add_network(obj)
        except KeyError, e:
            self._convert_to_validation_error(e, self.schema['network'])


@view_config(route_name='network_edit')
class Edit(gocept.directory2.admin.form.Edit):

    title = 'Edit Network'
    schema = Schema()
    next_route = 'network_view'
