import gocept.directory2.admin.testing
import transaction


class Authentication(gocept.directory2.admin.testing.BrowserTestCase):

    layer = gocept.directory2.admin.testing.BROWSER_LAYER

    def setUp(self):
        super(Authentication, self).setUp()
        from gocept.directory2.user import HumanUser
        HumanUser.create(
            uid='benutzer', firstname='Ben', lastname='Utzer', password='asdf',
            is_admin=True)
        transaction.commit()

    def login(self, user='benutzer', password='asdf'):
        b = self.browser
        b.getControl('Username').value = user
        b.getControl('Password').value = password
        b.getControl(name='login').click()

    def test_home_shows_login_form_which_does_login(self):
        b = self.browser
        b.open('http://localhost/')
        self.assertEqual('http://localhost/login', b.url)
        self.login()
        self.assertEqual('http://localhost/', b.url)
        self.assertIn('Logged in', b.contents)

    def test_view_redirects_to_login_form(self):
        b = self.browser
        b.open('http://localhost/machine/list')
        self.assertEqual('http://localhost/login', b.url)

    def test_logout_redirects_to_login_form_and_shows_message(self):
        b = self.browser
        b.open('http://localhost/')
        self.login()
        b.getLink('Logout').click()
        self.assertEqual('http://localhost/login', b.url)
        self.assertIn('Logged out', b.contents)

    def test_invalid_credentials_shows_login_form_with_error_msg(self):
        b = self.browser
        b.open('http://localhost/')
        self.login('foo', 'bar')
        self.assertEqual('http://localhost/login', b.url)
        self.assertIn('Login failed.', b.contents)

    def test_login_is_remembered(self):
        b = self.browser
        b.open('http://localhost/')
        self.login()
        b.open('http://localhost/')
        self.assertEqual('http://localhost/', b.url)

    def test_non_admin_user_cannot_access(self):
        from gocept.directory2.user import HumanUser
        HumanUser.create(
            uid='beliebig', firstname='B.', lastname='Liebig', password='asdf',
            is_admin=False)
        transaction.commit()
        b = self.browser
        b.open('http://localhost/')
        self.login('beliebig', 'asdf')
        self.assertEqual('http://localhost/login', b.url)
        self.assertIn('Logged in.', b.contents)

    def test_service_user_cannot_login(self):
        from gocept.directory2.user import ServiceUser
        ServiceUser.create(uid='service', password='asdf')
        transaction.commit()
        b = self.browser
        b.open('http://localhost/')
        self.login('service', 'asdf')
        self.assertEqual('http://localhost/login', b.url)
        self.assertIn('Login failed.', b.contents)
