from gocept.directory2.ipaddress import IPAddress
from gocept.directory2.machine import VirtualMachine
from gocept.directory2.network import Network
import gocept.directory2.admin.testing
import transaction


class TestIPAddress(gocept.directory2.admin.testing.BrowserTestCase):

    def setUp(self):
        super(TestIPAddress, self).setUp()
        self.vm = VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        location = self.layer['loc0']._refresh()
        network = Network.create(
            vlan=location.vlans['srv'], network='195.62.106.0/27')
        IPAddress.create(
            address='195.62.106.5', network=network, nic=self.vm.nics['srv'])
        transaction.commit()

    def test_edit_ipaddress(self):
        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('srv').click()
        b.getLink('195.62.106.5').click()
        self.assertEllipsis('...Used by...bar.srv...', b.contents)
        b.getLink('Edit reverse DNS').click()
        b.getControl('Reverse DNS name (optional)').value = 'example.com'
        b.getControl(name='save').click()
        self.assertEllipsis('...Updated on...', b.contents)
        self.assertEllipsis('...Reverse DNS name...example.com...', b.contents)

    def test_deallocate_ipaddress_removes_it_from_nic(self):
        self.assertEqual(1, len(self.vm.nics['srv'].addresses))
        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('srv').click()
        b.getLink('195.62.106.5').click()
        # XXX very unspecific selector, testbrowser can't seem to getControl()
        # the <button>Deallocate</button>, though.
        b.getForm(index=0).submit()
        self.assertEllipsis('...195.62.106.5...', b.contents)
        self.assertRaises(LookupError, lambda: b.getForm(index=0))
        self.vm._refresh()
        self.assertEqual(0, len(self.vm.nics['srv'].addresses))
