# -*- coding: utf-8 -*-
from gocept.directory2.location import Location
from gocept.directory2.network import Network
from mock import patch
import gocept.directory2.admin.testing
import transaction


class TestLocationCRUD(gocept.directory2.admin.testing.BrowserTestCase):
    """Testing ..location.*"""

    def test_location_list_displays_existing_locations(self):
        Location.create(name='foo')
        b = self.browser
        b.open('http://localhost/location/list')
        self.assertIn('foo', b.contents)

    def test_add_location(self):
        self.assertEqual(1, Location.query().count())
        browser = self.browser
        browser.open('http://localhost/location/list')
        browser.getLink('Add location').click()
        browser.getControl('Name').value = 'Development network'
        browser.getControl('Identifier').value = 'dev'
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        self.assertEqual(2, Location.query().count())
        location = Location.query().filter_by(abbreviation='dev').first()
        self.assertEqual('Development network', location.name)

    def test_view_location(self):
        Location.create(name='baz')
        browser = self.browser
        browser.open('http://localhost/location/list')
        browser.getLink('baz').click()
        self.assertEqual('http://localhost/location/2', browser.url)
        self.assertIn('baz', browser.contents)

    def test_edit_location(self):
        Location.create(name='baz', abbreviation='bar')
        transaction.commit()
        browser = self.browser
        browser.open('http://localhost/location/list')
        browser.getLink('baz').click()
        browser.getLink('Edit').click()
        # Displays stored values:
        self.assertEqual('baz', browser.getControl('Name').value)
        # Values can be changed:
        browser.getControl('Name').value = 'qux'
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        # Changed values get persisted:
        browser.getLink('Edit').click()
        self.assertEqual('qux', browser.getControl('Name').value)

    def test_delete_location(self):
        Location.create(name='qwe')
        browser = self.browser
        browser.open('http://localhost/location/list')
        browser.getLink('qwe').click()
        browser.getLink('Delete').click()
        self.assertIn(
            'Confirm deletion of location', browser.contents)
        browser.getControl(name='delete').click()
        self.assertEqual('http://localhost/location/list', browser.url)
        self.assertNotIn('qwe', browser.contents)

    def test_abbreviation_needs_to_be_ASCII_to_be_valid(self):
        browser = self.browser
        browser.open('http://localhost/location/add')
        browser.getControl('Short name / Identifier').value = 'Bär'
        browser.getControl(name='add').click()
        self.assertIn('"Bär" is not ASCII, which is required here.',
                      browser.contents)


class MailContactsTests(gocept.directory2.admin.testing.BrowserTestCase):
    """Testing ..location.MailContacts."""

    def test_sends_email(self):
        browser = self.browser
        browser.open('http://localhost/location/1')
        browser.getLink('Notify all contacts').click()
        self.assertEqual('http://localhost/location/1/mail', browser.url)
        browser.getControl('Subject').value = 'The end is near!'
        browser.getControl('Message').value = 'We will all die!'
        mail_contacts = 'gocept.directory2.location.Location.mail_contacts'
        with patch(mail_contacts) as mail_contacts:
            browser.getControl(name='send').click()
        mail_contacts.assert_called_with(
            u'The end is near!', u'We will all die!')
        self.assertEqual('http://localhost/location/1', browser.url)
        self.assertIn('Successfully sent message.', browser.contents)


class NetworkReplaceBrowserTest(
        gocept.directory2.admin.testing.BrowserTestCase):

    def test_replaces_network(self):
        location = self.layer['loc0']._refresh()
        Network.create(vlan=location.vlans['srv'], network='195.62.106.0/27')
        b = self.browser
        b.open('http://localhost/location/list')
        b.getLink('loc0').click()
        b.getLink('Networks').click()
        b.getLink('Replace network').click()
        b.getControl('Old network').displayValue = ['195.62.106.0/27']
        b.getControl('New network (CIDR)').value = '195.62.107.0/27'
        b.getControl(name='replace').click()
        self.assertNotIn('195.62.106.0/27', b.contents)
        self.assertEllipsis('...srv...195.62.107.0/27...', b.contents)
