from gocept.directory2.machine import VirtualMachine
from gocept.directory2.network import Network
import gocept.directory2.admin.testing
import transaction


class TestNICCRUD(gocept.directory2.admin.testing.BrowserTestCase):

    def test_view_and_edit_nic(self):
        VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('srv').click()
        self.assertEllipsis('...02:00:...', b.contents)
        b.getLink('Edit').click()
        b.getControl('MAC address').value = 'changed-mac'
        b.getControl(name='save').click()
        self.assertEllipsis('...Updated on...', b.contents)
        self.assertEllipsis('...changed-mac...', b.contents)

    def test_add_nic(self):
        VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('Add NIC').click()
        b.getControl('MAC address').value = 'my-mac'
        b.getControl('VLAN').displayValue = ['Storage']
        b.getControl(name='add').click()
        self.assertEllipsis('...added successfully...', b.contents)
        b.getLink('sto').click()
        self.assertEllipsis('...my-mac...', b.contents)

    def test_adding_nic_when_nic_for_that_vlan_already_exists_fails(self):
        vm = VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        transaction.commit()
        self.assertEqual(len(VirtualMachine.default_nics), len(vm.nics))
        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('Add NIC').click()
        b.getControl('MAC address').value = 'my-mac'
        b.getControl('VLAN').displayValue = ['Frontend']
        b.getControl(name='add').click()
        self.assertEllipsis('...already has a NIC for VLAN...', b.contents)
        self.assertEqual(len(VirtualMachine.default_nics), len(vm.nics))

    def test_allocate_address_without_index_chooses_one(self):
        VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        location = self.layer['loc0']._refresh()
        CIDR = '195.62.106.0/27'
        Network.create(vlan=location.vlans['srv'], network=CIDR)

        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('srv').click()
        b.getLink('Allocate address').click()
        b.getControl('Network').displayValue = [CIDR]
        b.getControl(name='allocate').click()
        self.assertEllipsis(
            '...Address allocated: 195.62.106.2...', b.contents)

    def test_allocate_address_with_index_uses_that(self):
        VirtualMachine.create(
            name='bar',
            location=self.layer['loc0'], resource_group=self.layer['rg0'])
        location = self.layer['loc0']._refresh()
        CIDR = '195.62.106.0/27'
        Network.create(vlan=location.vlans['srv'], network=CIDR)

        b = self.browser
        b.open('http://localhost/machine/list')
        b.getLink('bar').click()
        b.getLink('srv').click()
        b.getLink('Allocate address').click()
        b.getControl('Network').displayValue = [CIDR]
        b.getControl('Index').value = '5'
        b.getControl(name='allocate').click()
        self.assertEllipsis(
            '...Address allocated: 195.62.106.5...', b.contents)
