# -*- coding: utf-8 -*-
from gocept.directory2.permission import Permission
import gocept.directory2.admin.testing


class TestPermissionCRUD(gocept.directory2.admin.testing.BrowserTestCase):
    """Testing ..permission.*"""

    def test_permission_list_displays_existing_permissions(self):
        Permission.create(name='foo')
        b = self.browser
        b.open('http://localhost/permission/list')
        self.assertIn('foo', b.contents)

    def test_add_permission(self):
        self.assertEqual(0, Permission.query().count())
        browser = self.browser
        browser.open('http://localhost/permission/list')
        browser.getLink('Add permission').click()
        browser.getControl('Name').value = 'Code'
        browser.getControl('Numerical id').value = '27'
        browser.getControl(name='add').click()
        self.assertIn('added successfully', browser.contents)
        self.assertEqual(1, Permission.query().count())
        permission = Permission.query().first()
        self.assertEqual('Code', permission.name)

    def test_view_permission(self):
        Permission.create(name='baz')
        browser = self.browser
        browser.open('http://localhost/permission/list')
        browser.getLink('baz').click()
        self.assertEqual('http://localhost/permission/1', browser.url)
        self.assertIn('baz', browser.contents)

    def test_edit_permission(self):
        Permission.create(name='baz', id_number=28)
        browser = self.browser
        browser.open('http://localhost/permission/list')
        browser.getLink('baz').click()
        browser.getLink('Edit').click()
        # Displays stored values:
        # Name is a read only field
        with self.assertRaises(LookupError):
            browser.getControl('Name')
        self.assertIn('Name', browser.contents)
        self.assertIn('baz', browser.contents)
        # Values can be changed:
        browser.getControl('Numerical id').value = '29'
        browser.getControl(name='save').click()
        self.assertIn('Updated on', browser.contents)
        # Changed values get persisted:
        browser.getLink('Edit').click()
        self.assertEqual('29', browser.getControl('Numerical id').value)

    def test_delete_permission(self):
        Permission.create(name='qwe')
        browser = self.browser
        browser.open('http://localhost/permission/list')
        browser.getLink('qwe').click()
        browser.getLink('Delete').click()
        self.assertIn(
            'Confirm deletion of permission', browser.contents)
        browser.getControl(name='delete').click()
        self.assertEqual('http://localhost/permission/list', browser.url)
        self.assertNotIn('qwe', browser.contents)
