# -*- coding: utf-8 -*-
# Copyright (c) 2013 gocept gmbh & co. kg
# See also LICENSE.txt
from mock import patch, Mock
import gocept.directory2.admin.testing
import gocept.testing.assertion
import unittest
import datetime


class TestTrafficCRUD(gocept.directory2.admin.testing.BrowserTestCase):
    """Testing ..traffic.TrafficByCustomerDetail"""

    def setUp(self):
        super(TestTrafficCRUD, self).setUp()
        from gocept.directory2.resourcegroup import ResourceGroup
        from gocept.directory2.traffic import Traffic
        ResourceGroup.create(id=10, name='karlproject', customer_no=10353)
        ResourceGroup.create(id=11, name='osfkarl', customer_no=10353)
        ResourceGroup.create(id=12, name='sw83', customer_no=10468)
        Traffic.create(
            day=datetime.date(2013, 5, 29), resource_group_id=10,
            ipaddress='123.456.789.0', traffic=2048)
        Traffic.create(
            day=datetime.date(2013, 5, 29), resource_group_id=10,
            ipaddress='123.456.789.1', traffic=4096)
        Traffic.create(
            day=datetime.date(2013, 5, 30), resource_group_id=10,
            ipaddress='123.456.789.0', traffic=4096)
        Traffic.create(
            day=datetime.date(2013, 5, 29), resource_group_id=11, traffic=1024)
        Traffic.create(
            day=datetime.date(2013, 5, 29), resource_group_id=12, traffic=10)

    def test_lists_resource_groups_filtered_by_customer(self):
        browser = self.browser
        browser.open('http://localhost/report/traffic_by_customer')
        self.assertEllipsis("""...
<tr>
  <td>10353</td>
  <td>
    <a href="http://localhost/resourcegroup/10">karlproject</a>
    <a href="http://localhost/resourcegroup/11">osfkarl</a>
  </td>
  <td>
    <a href="http://localhost/report/traffic_by_customer/10353">Traffic</a>
  </td>
</tr>
<tr>
  <td>10468</td>
  <td>
      <a href="http://localhost/resourcegroup/12">sw83</a>
  </td>
  <td>
    <a href="http://localhost/report/traffic_by_customer/10468">Traffic</a>
  </td>
</tr>...""", browser.contents)

    def test_details_view_lists_traffic_for_each_rg_of_a_customer(self):
        browser = self.browser
        browser.open(
            'http://localhost/report/traffic_by_customer/10353?month=2013-05')
        self.assertEllipsis("""...
<tbody>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/10">karlproject</a>
    </td>
    <td style="text-align:right;">10 kB</td>
  </tr>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/11">osfkarl</a>
    </td>
    <td style="text-align:right;">1 kB</td>
  </tr>
  <tr>
    <th>Total</th>
    <td style="text-align:right;">11 kB</td>
  </tr>
</tbody>...""", browser.contents)

    def test_can_switch_to_previous_and_next_month(self):
        browser = self.browser
        browser.open(
            'http://localhost/report/traffic_by_customer/10353?month=2013-05')
        browser.getLink('Next month').click()
        self.assertTrue(browser.url.endswith('10353?month=2013-06'))
        browser.open(
            'http://localhost/report/traffic_by_customer/10353?month=2013-12')
        browser.getLink('Next month').click()
        self.assertTrue(browser.url.endswith('10353?month=2014-01'))
        browser.open(
            'http://localhost/report/traffic_by_customer/10353?month=2013-01')
        browser.getLink('Previous month').click()
        self.assertTrue(browser.url.endswith('10353?month=2012-12'))

    @patch('gocept.directory2.admin.traffic.ListTrafficByMonthForRG._last')
    def test_tab_on_resource_displays_traffic_of_last_4_months(self, _last):
        _last.return_value = datetime.date(2013, 5, 30)
        browser = self.browser
        browser.open('http://localhost/resourcegroup/10/traffic')
        self.assertEllipsis("""...
<tbody>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/10/traffic_detail?month=2013-02">
        2013-02</a>
    </td>
    <td>0 Bytes</td>
  </tr>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/10/traffic_detail?month=2013-03">
        2013-03</a>
    </td>
    <td>0 Bytes</td>
  </tr>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/10/traffic_detail?month=2013-04">
        2013-04</a>
    </td>
    <td>0 Bytes</td>
  </tr>
  <tr>
    <td>
      <a href="http://localhost/resourcegroup/10/traffic_detail?month=2013-05">
        2013-05</a>
    </td>
    <td>10 kB</td>
  </tr>
</tbody>...""", browser.contents)

    def test_details_view_for_resource_displays_total_traffic(self):
        browser = self.browser
        browser.open(
            'http://localhost/resourcegroup/10/traffic_detail?month=2013-05')
        self.assertEllipsis("""...
<p>Total traffic in this month: <span>10 kB</span></p>...""", browser.contents)

    def test_details_view_for_resource_displays_traffic_by_ipaddress(self):
        browser = self.browser
        browser.open(
            'http://localhost/resourcegroup/10/traffic_detail?month=2013-05')
        self.assertEllipsis("""...
<tbody>
  <tr>
    <td>123.456.789.0</td>
    <td>6 kB</td>
  </tr>
  <tr>
    <td>123.456.789.1</td>
    <td>4 kB</td>
  </tr>
</tbody>...""", browser.contents)

    def test_details_view_for_resource_displays_traffic_by_day(self):
        browser = self.browser
        browser.open(
            'http://localhost/resourcegroup/10/traffic_detail?month=2013-05')
        self.assertEllipsis("""...
<tbody>
  <tr>
    <td>2013-05-29</td>
    <td>6 kB</td>
  </tr>
  <tr>
    <td>2013-05-30</td>
    <td>4 kB</td>
  </tr>
</tbody>...""", browser.contents)


class TestFormatTrafficFromInt(unittest.TestCase):
    """Testing ..traffic.format_traffic_from_int."""

    def test_bytes_if_smaller_than_one_kb(self):
        from ..traffic import format_traffic_from_int
        self.assertEqual('0 Bytes', format_traffic_from_int(0))
        self.assertEqual('1 Byte', format_traffic_from_int(1))
        self.assertEqual('2 Bytes', format_traffic_from_int(2))
        self.assertEqual('1023 Bytes', format_traffic_from_int(1023))

    def test_rounded_kb_if_smaller_that_one_mb_and_greater_than_1023_b(self):
        from ..traffic import format_traffic_from_int
        self.assertEqual('1 kB', format_traffic_from_int(1024))
        self.assertEqual('1 kB', format_traffic_from_int(1200))
        self.assertEqual('2 kB', format_traffic_from_int(2048))
        self.assertEqual('1023 kB', format_traffic_from_int(1024 * 1024 - 1))

    def test_rounded_mb_if_smaller_that_one_gib_and_greater_than_1023_kb(self):
        from ..traffic import format_traffic_from_int
        self.assertEqual('1 MB', format_traffic_from_int(1024 * 1024))
        self.assertEqual('1 MB', format_traffic_from_int(1024 * 1024 * 2 - 1))
        self.assertEqual('2 MB', format_traffic_from_int(1024 * 1024 * 2))
        self.assertEqual(
            '1023 MB', format_traffic_from_int(1024 * 1024 * 1024 - 1))

    def test_rounded_gib_if_greater_than_1023_mb(self):
        from ..traffic import format_traffic_from_int
        self.assertEqual('1 GiB', format_traffic_from_int(1024 * 1024 * 1024))
        self.assertEqual(
            '1 GiB', format_traffic_from_int(1024 * 1024 * 1024 * 2 - 1))
        self.assertEqual(
            '2 GiB', format_traffic_from_int(1024 * 1024 * 1024 * 2))
        self.assertEqual(
            '1023 GiB', format_traffic_from_int(1024 * 1024 * 1024 * 1024 - 1))
        self.assertEqual(
            '2048 GiB', format_traffic_from_int(1024 * 1024 * 1024 * 1024 * 2))


class TestListTrafficByMonthForRGUnitTests(unittest.TestCase):
    """ ..traffic.ListTrafficByMonthForRG."""

    @patch('gocept.directory2.admin.traffic.ListTrafficByMonthForRG._last')
    def test_first_returns_first_of_month_four_months_before_last(self, _last):
        from ..traffic import ListTrafficByMonthForRG
        view = ListTrafficByMonthForRG(Mock())
        _last.return_value = datetime.date(2013, 4, 29)
        self.assertEqual(datetime.date(2013, 1, 1), view._first())
        _last.return_value = datetime.date(2013, 5, 1)
        self.assertEqual(datetime.date(2013, 2, 1), view._first())
        _last.return_value = datetime.date(2013, 2, 28)
        self.assertEqual(datetime.date(2012, 11, 1), view._first())
        _last.return_value = datetime.date(2013, 3, 31)
        self.assertEqual(datetime.date(2012, 12, 1), view._first())

    @patch('gocept.directory2.admin.traffic.ListTrafficByMonthForRG._last')
    def test_months_returns_list_of_last_4_months(self, _last):
        from ..traffic import ListTrafficByMonthForRG
        view = ListTrafficByMonthForRG(Mock())
        _last.return_value = datetime.date(2013, 4, 29)
        self.assertEqual(['2013-01', '2013-02', '2013-03', '2013-04'],
                         list(view.months()))
        _last.return_value = datetime.date(2013, 1, 1)
        self.assertEqual(['2012-10', '2012-11', '2012-12', '2013-01'],
                         list(view.months()))
        _last.return_value = datetime.date(2013, 1, 31)
        self.assertEqual(['2012-10', '2012-11', '2012-12', '2013-01'],
                         list(view.months()))
