from ..user import User, HumanUser
from .resourcegroup import ResourceGroupTabs
from .schema import ReadOnlyNode
from .view import View, Action
from colander import SchemaNode, String, Boolean
from deform.widget import SelectWidget
from pyramid.view import view_config
import colander
import gocept.directory2.admin.form

AVAILABLE_LOGIN_SHELLS = [
    '/bin/bash', '/bin/zsh', '/sbin/nologin', '/bin/false']


@view_config(route_name='user_list',
             renderer='templates/user/list.pt')
class ListUsers(View):

    title = 'Users'
    actions = [
        Action('Add human user', 'icon-plus', 'user_add_human'),
        Action('Add service user', 'icon-plus', 'user_add_service'),
    ]

    @property
    def users(self):
        return User.query()


class AddHumanUserSchema(colander.Schema):
    """Schema to add a human user."""

    uid = SchemaNode(String(), title='Login name')
    login_shell = SchemaNode(
        String(), title='Login shell', widget=SelectWidget(
            values=[(s, s) for s in AVAILABLE_LOGIN_SHELLS]),
        default='/bin/bash')
    # XXX: SSH keys
    mail = SchemaNode(String(), title='E-mail address')
    pager = SchemaNode(String(), missing='')
    firstname = SchemaNode(String())
    lastname = SchemaNode(String())
    is_admin = SchemaNode(Boolean(), missing=False)
    description = SchemaNode(String(), missing='')


@view_config(route_name='user_add_human')
class AddHuman(gocept.directory2.admin.form.Add):
    """Add a new human user."""

    title = 'Add human user'
    schema = AddHumanUserSchema()
    factory = HumanUser
    next_route = 'user_list'


class EditHumanUserSchema(AddHumanUserSchema):

    uid = ReadOnlyNode(String(), title='Login name')
    password = SchemaNode(String(), missing='')  # XXX: password field
    # XXX confirm_password -> #12370


@view_config(route_name='user_edit_human')
class EditHuman(gocept.directory2.admin.form.Edit):
    """Add a new human user."""

    title = 'Edit human user'
    schema = EditHumanUserSchema()
    next_route = 'user_list'


@view_config(route_name='user_view_human',
             renderer='templates/user/view_human.pt')
class UserViewHuman(View):

    @property
    def title(self):
        return self.context.name

    @property
    def actions(self):
        yield Action(
            'Edit', 'icon-edit', 'user_edit_human', id=self.context.id)
        yield Action(
            'Delete', 'icon-remove', 'user_confirm_delete', id=self.context.id)


@view_config(route_name='user_confirm_delete')
class ConfirmDelete(gocept.directory2.admin.form.ConfirmDelete):
    """Confirm deletion of user."""

    delete_route = 'user_delete'

    @property
    def subject(self):
        return 'User "%s"' % self.context.name


@view_config(route_name='user_delete')
class Delete(gocept.directory2.admin.form.Delete):
    """Actually delete user."""

    next_route = 'user_list'


@view_config(route_name='resourcegroup_users',
             renderer='templates/resourcegroup/users.pt')
class ListUsersForRG(ResourceGroupTabs, View):
    """List users for resource group."""

    def __call__(self):
        for user_id in self.request.POST.getall('users'):
            user = User.query().get(user_id)
            if self.request.POST.get('add'):
                self.context.users.add(user)
            elif self.request.POST.get('remove'):
                self.context.users.remove(user)
        return super(ListUsersForRG, self).__call__()

    @property
    def title(self):
        return self.context.name

    def _info(self, user):
        user_type = type(user).__name__
        route_name = 'user_view_%s' % (
            'human' if user_type == 'HumanUser' else 'service')
        return {'id': user.id,
                'login_name': user.uid,
                'type': user_type,
                'route_name': route_name}

    def users(self):
        for user in self.context.users:
            yield self._info(user)

    def all_users(self):
        for user in User.query().all():
            if user not in self.context.users:
                yield self._info(user)
