#encoding=utf8
import gocept.directory2.admin.resources


class Data(object):
    """Data object."""

    def __init__(self, **kw):
        self.__dict__.update(kw)


class Action(object):
    """Action the user can trigger."""

    def __init__(self, title, css_class, route, **route_kw):
        self.title = title
        self.css_class = css_class
        self.route = route
        self.route_kw = route_kw

    def __call__(self, request):
        return Data(url=request.route_url(self.route, **self.route_kw),
                    title=self.title,
                    css_class=self.css_class,
                    active=(self.route == request.matched_route.name))


class View(object):
    """Base class for all views to integrate view with main template."""

    # Values for display above view contents (optional):
    title = None
    subtitle = None
    description = None

    # Sequence of Action objects (see above.)
    actions = ()
    tabs = ()

    def __init__(self, request):
        gocept.directory2.admin.resources.gocept_directory2.need()
        self.request = request
        self.context = request.context
        # Storing self to access attributes in main template:
        request.content_view = self

    @property
    def action_data(self):
        return [action(self.request) for action in self.actions]

    @property
    def tab_data(self):
        return [tab(self.request) for tab in self.tabs]

    @property
    def content_css_class(self):
        if self.actions:
            return 'span9'
        return 'span12'

    def __call__(self):
        # Access the view attributes using view.attribute in the template.
        return {}


def pimp_my_boolean(value):
    # Display nice UI elements for boolean values.
    return u'<span class="bool true">✓</span>' if value \
        else u'<span class="bool false">✗</span>'
