from ..machine import Node
from ..resourcegroup import ResourceGroup
from .utils import node_to_dict
from pyramid.view import view_config
import pyramid.security
import pyramid_xmlrpc


class RootFactory(object):
    """Traversal root factory to provide ACL for Ring 1.

    Context for the Ring1 view.

    """
    def __init__(self, request):
        self.request = request

    @property
    def resource_group(self):
        """Name of the resource group requested via URL."""
        return self.request.matchdict['resource_group']

    @property
    def __acl__(self):
        return [(pyramid.security.Allow,
                 'group:{}'.format(self.resource_group),
                 'access')]


@view_config(route_name='ring1', permission='access')
class Ring1(pyramid_xmlrpc.XMLRPCView):

    @property
    def _resource_group(self):
        # The resource group limiting this view.
        return ResourceGroup.query().filter(
            ResourceGroup.name == self.context.resource_group).one()

    def lookup_node(self, lookup_name):
        node = Node.query().filter(Node.resource_group == self._resource_group,
                                   Node.name == lookup_name).one()
        return node_to_dict(node)
