import gocept.directory2.db
import grokcore.component.zcml
import martian
import pyramid.config
import pyramid_beaker
import re
import zope.component
import zope.configuration.config
import zope.configuration.xmlconfig


class ApplicationFactory(object):
    """This application factory contains the general code needed to assemble
    the software environment for the directory, independent of the interface or
    scenario: the business model is made functional here as well as commonly
    used code to create Pyramid and WSGI configuration.

    Specific interfaces should sub-class this and use the following methods to
    extend the behaviour:

    configure() to extend the Pyramid application
    make_wsgi_app() to extend the WSGI pipeline

    If you call the application factory then a full-fledged, production-ready
    instance with full runtime configuration will be returned

    If you want to use an application for testing purposes, then you can call
    configure() and make_wsgi_app() manually.

    """

    DONT_GROK = (
        'scripts',
        'tests',
        'testing',
        'admin',
        'api',
    )

    def __init__(self, **settings):
        self.settings = settings

    def __call__(self, global_config, **settings):
        self.settings = settings
        self.configure()
        self.setup_runtime()
        self.check_db_revision()
        return self.make_wsgi_app()

    def configure(self):
        self.__grok()
        self.config = config = pyramid.config.Configurator(
            settings=self.settings,
            registry=zope.component.getGlobalSiteManager())
        config.setup_registry(settings=self.settings)
        config.set_session_factory(
            pyramid_beaker.BeakerSessionFactoryConfig(
                type='memory',  # XXX improve before deployment
                lock_dir='beaker.lock_dir'))

        config.include('pyramid_tm')
        config.include('pyramid_mailer')

    def setup_runtime(self):
        # XXX This should actually be controlled from even further outside ...
        from .db import Database
        from .db_domainshuttle import Database as DomainDatabase
        db = Database(self.settings['sqlalchemy.url'])
        db_ds = DomainDatabase(self.settings['sqlalchemy.domainshuttle_url'])
        zope.component.provideUtility(db)
        zope.component.provideUtility(db_ds)
        try:
            db.session.execute('SELECT * from tmp_functest')
        except:
            pass
        else:
            raise ValueError(
                "Possibly running against a testing database.")

    def check_db_revision(self):
        db = zope.component.getUtility(gocept.directory2.db.IDatabase)
        db.assert_database_revision_is_current()

    def make_wsgi_app(self):
        return self.config.make_wsgi_app()

    def __grok(self):
        context = zope.configuration.config.ConfigurationMachine()
        zope.configuration.xmlconfig.registerCommonDirectives(context)
        # load zope.component configuration to make events work
        zope.configuration.xmlconfig.include(context, package=zope.component)
        grokcore.component.zcml.do_grok('grokcore.component.meta', context)
        martian.grok_dotted_name(
            'gocept.directory2',
            grokcore.component.zcml.the_module_grokker,
            exclude_filter=lambda name: name in set(self.DONT_GROK),
            config=context)
        context.execute_actions()


scan_ignore_tests = [re.compile(r'\.testing$').search,
                     re.compile('\.tests\.').search]
