from sqlalchemy import Column, String
import gocept.directory2.db
from gocept.directory2.mail import sendmail


class Location(gocept.directory2.db.Object):

    __tablename__ = 'locations'
    name = Column(String, unique=True)
    abbreviation = Column(String)

    MAIL_TEMPLATE = u"""%(content)s

We are notifying you because you are registered as the technical contact for
these nodes: %(nodes)s
"""

    def mail_contacts(self, subject, content):
        recipients = {}
        for node in self.nodes:
            for recipient in node.resource_group.technical_contacts:
                affected_nodes = recipients.setdefault(recipient, [])
                affected_nodes.append(node.name)
        recipients['admin@gocept.com'] = ['(informational copy)']
        subject = '[%s] %s' % (self.name, subject)

        for recipient, nodes in recipients.items():
                recipient_content = self.MAIL_TEMPLATE % dict(
                    content=content, nodes=', '.join(sorted(nodes)))
                sendmail(self.name, [recipient], 'en',
                         subject, recipient_content)
