from ..machine import Node
from ..network import Network
import gocept.directory2.testing
import gocept.testing.assertion
import transaction
import unittest


class NICEventsTest(unittest.TestCase,
                    gocept.testing.assertion.String):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    VLAN = 'sto'
    node_id = 0

    def create_node(self, **kw):
        self.node_id += 1
        return Node.create(
            name='foo%s' % self.node_id,
            location=self.layer['loc0'], resource_group=self.layer['rg0'],
            default_nics=[self.VLAN],
            **kw)

    def test_node_added_creates_default_nics(self):
        node = self.create_node()
        self.assertEqual(1, len(node.nics))
        self.assertEqual(self.VLAN, node.nics.values()[0].name)

    def test_generates_mac_if_enabled(self):
        node = self.create_node()
        self.assertEqual(None, node.nics[self.VLAN].mac)
        node = self.create_node(generate_default_mac=True)
        self.assertStartsWith('02:', node.nics[self.VLAN].mac)

    def test_allocates_ipv6_addresses(self):
        location = self.layer['loc0']._refresh()
        vlan = location.vlans[self.VLAN]
        Network.create(
            network='2002:470:9aaf:4%s::/64' % vlan.vlanclass.number,
            vlan=vlan)
        node = self.create_node()
        self.assertStartsWith('2002', node.nics[self.VLAN].addresses[0].cidr)

    def test_allocated_addresses_are_freed_when_nic_is_deleted(self):
        location = self.layer['loc0']._refresh()
        vlan = location.vlans[self.VLAN]
        Network.create(
            network='2002:470:9aaf:4%s::/64' % vlan.vlanclass.number,
            vlan=vlan)
        node = self.create_node()
        address = node.nics[self.VLAN].addresses[0]
        node.nics[self.VLAN].delete()
        transaction.commit()
        address._refresh()
        self.assertEqual(None, address.nic)
